# ********************************************************************************
# *         WARNING: This file is automatically generated by unasync.py.         *
# *                             DO NOT MANUALLY EDIT                             *
# *           Source File: patent_client/_async/uspto/ptab/api_test.py           *
# ********************************************************************************

from .api import PtabApi


class TestPtabApi:
    def test_can_get_document(self):
        page = PtabApi.get_documents(document_identifier="170680401")
        assert len(page.docs) == 1

    def test_can_get_document_range(self):
        page = PtabApi.get_documents(
            proceeding_number="IPR2022-00037",
            document_filing_date=("2023-01-01", "2023-09-15"),
        )
        assert len(page.docs) == 25

    def test_can_get_proceeding(self):
        page = PtabApi.get_proceedings(proceeding_number="IPR2022-00037")
        assert len(page.docs) == 1
