# ********************************************************************************
# *         WARNING: This file is automatically generated by unasync.py.         *
# *                             DO NOT MANUALLY EDIT                             *
# *                   Source File: patent_client/_async/odp.py                   *
# ********************************************************************************

# A simple separate namespace for the USPTO's Open Data Portal.
from patent_client._sync.uspto.odp.model import USApplication, USApplicationBiblio

__all__ = ["USApplication", "USApplicationBiblio"]
