# ********************************************************************************
# *         WARNING: This file is automatically generated by unasync.py.         *
# *                             DO NOT MANUALLY EDIT                             *
# *            Source File: patent_client/_async/epo/ops/__init__.py             *
# ********************************************************************************

from .family.model import Family
from .legal.model import Legal
from .published.model import Images, Inpadoc

__api_name__ = "EPO Open Patent Services"
__all__ = ["Family", "Legal", "Images", "Inpadoc", "session"]
