# ********************************************************************************
# *         WARNING: This file is automatically generated by unasync.py.         *
# *                             DO NOT MANUALLY EDIT                             *
# *      Source File: patent_client/_async/uspto/public_search/api_test.py       *
# ********************************************************************************

from .api import PublicSearchApi


def test_simple_search():
    api = PublicSearchApi()
    results = api.run_query(
        query='("6013599").pn.',
    )
    assert results.num_found == 1
    assert results.docs[0].publication_number == "6013599"
