# ********************************************************************************
# *         WARNING: This file is automatically generated by unasync.py.         *
# *                             DO NOT MANUALLY EDIT                             *
# *           Source File: patent_client/_async/uspto/ptab/__init__.py           *
# ********************************************************************************

__api_name__ = "PTAB API v2"
from .model import PtabDecision, PtabDocument, PtabProceeding  # noqa: F401,E402

__all__ = [
    "PtabDecision",
    "PtabDocument",
    "PtabProceeding",
]
